use serde::Serialize;

use crate::convert::map_cam16jch_to_srgb_hex;

use super::tonal_palette::TonalPalette;

#[derive(Debug, Clone, Serialize)]
pub struct M3ColorSet {
    pub root: String,
    pub on_root: String,
    pub container: String,
    pub on_conatiner: String,
    pub fixed: String,
    pub fixed_dim: String,
    pub on_fixed: String,
    pub fixed_variant: String,
    pub inverse: String,
}

impl M3ColorSet {
    pub fn new_light_set(palette: &TonalPalette) -> Self {
        let root = palette.tone(40.0);
        let on_root = palette.tone(100.0);
        let container = palette.tone(90.0);
        let on_container = palette.tone(30.0);
        let fixed = palette.tone(90.0);
        let fixed_dim = palette.tone(80.0);
        let on_fixed = palette.tone(10.0);
        let fixed_variant = palette.tone(30.0);
        let inverse = palette.tone(80.0);

        Self {
            root: map_cam16jch_to_srgb_hex(&root),
            on_root: map_cam16jch_to_srgb_hex(&on_root),
            container: map_cam16jch_to_srgb_hex(&container),
            on_conatiner: map_cam16jch_to_srgb_hex(&on_container),
            fixed: map_cam16jch_to_srgb_hex(&fixed),
            fixed_dim: map_cam16jch_to_srgb_hex(&fixed_dim),
            on_fixed: map_cam16jch_to_srgb_hex(&on_fixed),
            fixed_variant: map_cam16jch_to_srgb_hex(&fixed_variant),
            inverse: map_cam16jch_to_srgb_hex(&inverse),
        }
    }

    pub fn new_dark_set(palette: &TonalPalette) -> Self {
        let root = palette.tone(80.0);
        let on_root = palette.tone(20.0);
        let container = palette.tone(30.0);
        let on_container = palette.tone(90.0);
        let fixed = palette.tone(90.0);
        let fixed_dim = palette.tone(80.0);
        let on_fixed = palette.tone(10.0);
        let fixed_variant = palette.tone(30.0);
        let inverse = palette.tone(40.0);

        Self {
            root: map_cam16jch_to_srgb_hex(&root),
            on_root: map_cam16jch_to_srgb_hex(&on_root),
            container: map_cam16jch_to_srgb_hex(&container),
            on_conatiner: map_cam16jch_to_srgb_hex(&on_container),
            fixed: map_cam16jch_to_srgb_hex(&fixed),
            fixed_dim: map_cam16jch_to_srgb_hex(&fixed_dim),
            on_fixed: map_cam16jch_to_srgb_hex(&on_fixed),
            fixed_variant: map_cam16jch_to_srgb_hex(&fixed_variant),
            inverse: map_cam16jch_to_srgb_hex(&inverse),
        }
    }

    pub fn to_css_variables(&self, prefix: &str, name: &str) -> Vec<String> {
        let mut variable_lines = Vec::new();

        variable_lines.push(format!("--color-{}-{}: #{};", prefix, name, self.root));
        variable_lines.push(format!(
            "--color-{}-on-{}: #{};",
            prefix, name, self.on_root
        ));
        variable_lines.push(format!(
            "--color-{}-{}-container: #{};",
            prefix, name, self.container
        ));
        variable_lines.push(format!(
            "--color-{}-on-{}-container: #{};",
            prefix, name, self.on_conatiner
        ));
        variable_lines.push(format!(
            "--color-{}-{}-fixed: #{};",
            prefix, name, self.fixed
        ));
        variable_lines.push(format!(
            "--color-{}-{}-fixed-dim: #{};",
            prefix, name, self.fixed_dim
        ));
        variable_lines.push(format!(
            "--color-{}-on-{}-fixed: #{};",
            prefix, name, self.on_fixed
        ));
        variable_lines.push(format!(
            "--color-{}-on-{}-fixed-variant: #{};",
            prefix, name, self.fixed_variant
        ));
        variable_lines.push(format!(
            "--color-{}-inverse-{}: #{};",
            prefix, name, self.inverse
        ));

        variable_lines
    }

    pub fn to_scss_variables(&self, prefix: &str, name: &str) -> Vec<String> {
        let mut variable_lines = Vec::new();

        variable_lines.push(format!("$color-{}-{}: #{};", prefix, name, self.root));
        variable_lines.push(format!("$color-{}-on-{}: #{};", prefix, name, self.on_root));
        variable_lines.push(format!(
            "$color-{}-{}-container: #{};",
            prefix, name, self.container
        ));
        variable_lines.push(format!(
            "$color-{}-on-{}-container: #{};",
            prefix, name, self.on_conatiner
        ));
        variable_lines.push(format!(
            "$color-{}-{}-fixed: #{};",
            prefix, name, self.fixed
        ));
        variable_lines.push(format!(
            "$color-{}-{}-fixed-dim: #{};",
            prefix, name, self.fixed_dim
        ));
        variable_lines.push(format!(
            "$color-{}-on-{}-fixed: #{};",
            prefix, name, self.on_fixed
        ));
        variable_lines.push(format!(
            "$color-{}-on-{}-fixed-variant: #{};",
            prefix, name, self.fixed_variant
        ));
        variable_lines.push(format!(
            "$color-{}-inverse-{}: #{};",
            prefix, name, self.inverse
        ));

        variable_lines
    }

    pub fn to_javascript_object_fields(&self, prefix: &str, name: &str) -> Vec<String> {
        let mut variable_lines = Vec::new();
        let prefix = prefix.to_ascii_lowercase();
        let name = name
            .chars()
            .next()
            .unwrap()
            .to_ascii_uppercase()
            .to_string()
            + &name[1..];

        variable_lines.push(format!("{}{}: '#{}',", prefix, name, self.root));
        variable_lines.push(format!("{}on{}: '#{}',", prefix, name, self.on_root));
        variable_lines.push(format!(
            "{}{}Container: '#{}',",
            prefix, name, self.container
        ));
        variable_lines.push(format!(
            "{}On{}Container: '#{}',",
            prefix, name, self.on_conatiner
        ));
        variable_lines.push(format!("{}{}Fixed: '#{}',", prefix, name, self.fixed));
        variable_lines.push(format!(
            "{}{}FixedDim: '#{}',",
            prefix, name, self.fixed_dim
        ));
        variable_lines.push(format!("{}On{}Fixed: '#{}',", prefix, name, self.on_fixed));
        variable_lines.push(format!(
            "{}On{}FixedVariant: '#{}',",
            prefix, name, self.fixed_variant
        ));
        variable_lines.push(format!("{}Inverse{}: '#{}',", prefix, name, self.inverse));

        variable_lines
    }
}
