import cx from 'clsx';
import { useAtom } from 'jotai';
import { useMemo, useState } from 'react';
import { useColorFunction } from '../ColorFunctionContext';
import { ColorPicker } from '../components/ColorPicker';
import { VSegmentedControl } from '../components/VSegmentedControl';
import { HarmonyColor } from '../models';
import { HarmonyPreview } from '../page-components/harmonies/HarmonyPreview';
import { currentPickedColor } from '../stores/colors';
import styles from './Harmonies.module.css';

export function Harmonies() {
  const { colorFn } = useColorFunction();
  const [selectedColor, setSelectedColor] = useAtom(currentPickedColor);
  const [selectedMode, setSelectedMode] = useState('complementary');
  const generatedColors = useMemo(() => {
    try {
      const colors: HarmonyColor[] = [];
      switch (selectedMode) {
        case 'complementary':
          {
            colors.push({ color: selectedColor, ratio: 65 });
            const compColor = colorFn?.complementary(selectedColor) ?? '000000';
            colors.push({ color: compColor, ratio: 35 });
          }
          break;
        case 'analogous':
          {
            colors.push({ color: selectedColor, ratio: 60 });
            const analogousColors = colorFn?.analogous_30(selectedColor) ?? [];
            colors.push({ color: analogousColors[0], ratio: 30 });
            colors.push({ color: analogousColors[1], ratio: 10 });
          }
          break;
        case 'analogous_with_complementary':
          {
            colors.push({ color: selectedColor, ratio: 55 });
            const analogousColors = colorFn?.analogous_30(selectedColor) ?? [];
            colors.push({ color: analogousColors[0], ratio: 35 });
            colors.push({ color: analogousColors[1], ratio: 10 });
            const compColor = colorFn?.complementary(selectedColor) ?? '000000';
            colors.push({ color: compColor, ratio: 10 });
          }
          break;
        case 'triadic':
          {
            colors.push({ color: selectedColor, ratio: 60 });
            const triadicColors = colorFn?.triadic(selectedColor) ?? [];
            colors.push({ color: triadicColors[0], ratio: 30 });
            colors.push({ color: triadicColors[1], ratio: 10 });
          }
          break;
        case 'split_complementary':
          {
            colors.push({ color: selectedColor, ratio: 60 });
            const splitCompColors = colorFn?.split_complementary(selectedColor) ?? [];
            colors.push({ color: splitCompColors[0], ratio: 30 });
            colors.push({ color: splitCompColors[1], ratio: 10 });
          }
          break;
        case 'tetradic':
          {
            colors.push({ color: selectedColor, ratio: 55 });
            const tetradicColors = colorFn?.tetradic(selectedColor) ?? [];
            colors.push({ color: tetradicColors[0], ratio: 35 });
            colors.push({ color: tetradicColors[1], ratio: 10 });
            colors.push({ color: tetradicColors[2], ratio: 10 });
          }
          break;
        case 'flip_tetradic':
          {
            colors.push({ color: selectedColor, ratio: 55 });
            const color120 = colorFn?.shift_hue(selectedColor, 120) ?? '000000';
            colors.push({ color: color120, ratio: 35 });
            const compColor = colorFn?.complementary(selectedColor) ?? '000000';
            colors.push({ color: compColor, ratio: 10 });
            const color240 = colorFn?.shift_hue(selectedColor, 240) ?? '000000';
            colors.push({ color: color240, ratio: 10 });
          }
          break;
        case 'square':
          {
            colors.push({ color: selectedColor, ratio: 55 });
            const color90 = colorFn?.shift_hue(selectedColor, 90) ?? '000000';
            colors.push({ color: color90, ratio: 35 });
            const compColor = colorFn?.complementary(selectedColor) ?? '000000';
            colors.push({ color: compColor, ratio: 10 });
            const color270 = colorFn?.shift_hue(selectedColor, 270) ?? '000000';
            colors.push({ color: color270, ratio: 10 });
          }
          break;
        default:
          break;
      }
      return colors;
    } catch (error) {
      console.error('[Generate Color Series]', error);
    }
    return [] as HarmonyColor[];
  }, [selectedColor, selectedMode]);

  return (
    <div className={cx('workspace', styles.harmonies_workspace)}>
      <header>
        <h3>Color Harmonies</h3>
        <p>Automatically select color combinations using color theory.</p>
      </header>
      <section className={styles.explore_section}>
        <aside className={styles.function_side}>
          <div>
            <h5>Basic color</h5>
            <ColorPicker color={selectedColor} onSelect={(color) => setSelectedColor(color)} />
          </div>
          <div className={styles.mode_navigation}>
            <h5>Color selection method</h5>
            <VSegmentedControl
              onChange={setSelectedMode}
              options={[
                { label: 'Complementary', value: 'complementary' },
                { label: 'Analogous', value: 'analogous' },
                { label: 'Analogous with Complementary', value: 'analogous_with_complementary' },
                { label: 'Triadic', value: 'triadic' },
                { label: 'Split Complementary', value: 'split_complementary' },
                { label: 'Tetradic', value: 'tetradic' },
                { label: 'Flip Tetradic', value: 'flip_tetradic' },
                { label: 'Square', value: 'square' },
              ]}
            />
          </div>
        </aside>
        <div className={styles.preview_side}>
          <HarmonyPreview colors={generatedColors} />
        </div>
      </section>
    </div>
  );
}
